/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package JCSP.Encryption;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.params.CryptParamsInterface;
import ru.CryptoPro.JCP.params.CryptParamsSpec;
import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.JCSP.Key.GostPrivateKey;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import java.security.*;

/**
 * В данном примере производится зашифрование и расшифрование текста на
 * симметричном ключе. Для передачи этого ключа произоводится его зашифрование и
 * расшифрование на ключах согласования сторон с помощью эфемерных пар ключей.
 */
public class SessionEncryptByEph extends Common {

    protected String getAlicaAlias() {
        return "sebe_alica";
    }

    protected char[] getAlicaPassword() {
        return "sebe_alica".toCharArray();
    }

    protected String getBobAlias() {
        return "sebe_bob";
    }

    protected char[] getBobPassword() {
        return "sebe_bob".toCharArray();
    }

    /**
     * Выполнение шифрования.
     *
     * @param pairProvider Имя провайдера для генерации ключевых пар.
     * @param agreeProvider Провайдер для формирования ключей согласования.
     * @param simProvider Провайдер для генерации симметричного ключа.
     * @param simParams Параметры для сессионного ключа.
     * @param exchKeyAlgorithm Алгоритм шифрования или ключа обмена.
     * @param exchKeyAgreeAlgorithm Алгоритм для ключей согласования.
     * @param signAlgorithm Алгоритм подписи запроса.
     * @param create true, если следует создать контейнеры и сохранить их;
     * иначе предполагается, что они существуют и их нужно загрузить.
     * @param delete true, если по завершении следует удалить контейнер.
     * @throws Exception
     */
    @Override
    public void execute(String pairProvider, String agreeProvider,
        String simProvider, CryptParamsInterface simParams, String exchKeyAlgorithm,
        String exchKeyAgreeAlgorithm, String signAlgorithm, boolean create,
        boolean delete) throws Exception {

        System.out.println("Session encryption and decryption example.");
        byte[] data = text.getBytes();

        try {

            prepare(pairProvider, exchKeyAlgorithm, signAlgorithm, create, create);

            /* Генерация начальной синхропосылки для выработки
            ключа согласования */

            byte[] sv = new byte[randomLength];
            SecureRandom random = SecureRandom.getInstance(
                randomAlgorithmName, defaultProvider);
            random.nextBytes(sv);

            IvParameterSpec ivspec = new IvParameterSpec(sv);
            System.out.println("Syncro for KeyAgreement was generated");

            /* Выработка ключа согласования алисы со сгенеренным SV */

            KeyAgreement alicaKeyAgree =
                KeyAgreement.getInstance(exchKeyAgreeAlgorithm, agreeProvider);
            alicaKeyAgree.init(alicaPrivate, ivspec, null);
            alicaKeyAgree.doPhase(bobPublic, true);

            SecretKey alisaAgree =
                alicaKeyAgree.generateSecret(MacAlgorithm);
            System.out.println("Alica's key agreement was performed");

            /* Генерация симметричного ключа алисой с параметрами
            шифрования из контрольной панели. */

            KeyGenerator keyGen =
                KeyGenerator.getInstance(MacAlgorithm, simProvider);

            if (simParams != null) {
                keyGen.init(simParams);
            } // if

            SecretKey simKey = keyGen.generateKey();
            System.out.println("Session key was generated by alica");

            /* Зашифрование текста на симметричном ключе алисы */

            Cipher cipher = Cipher.getInstance(CipherAlgorithm, defaultProvider);
            cipher.init(Cipher.ENCRYPT_MODE, simKey);

            // передача вектора инициализации бобу
            byte[] iv = cipher.getIV();
            byte[] encryptedText = cipher.doFinal(data, 0, data.length);
            System.out.println("Alica's text encrypting was performed");

            /* Зашифрование симметричного ключа на ключе согласования алисы. */

            cipher.init(Cipher.WRAP_MODE, alisaAgree);
            byte[] wrappedKey = cipher.wrap(simKey);
            System.out.println("Alica's session key wrapping was performed");

            /* Выработка ключа согласования боба с тем же SV. */

            KeyAgreement bobKeyAgree =
                KeyAgreement.getInstance(exchKeyAgreeAlgorithm, agreeProvider);
            bobKeyAgree.init(bobPrivate, ivspec, null);
            bobKeyAgree.doPhase(alicaPublic, true);

            SecretKey bobAgree = bobKeyAgree.generateSecret(MacAlgorithm);
            System.out.println("Bob's key agreement was performed");

            /* Расшифрование бобом симметричного ключа. */

            cipher.init(Cipher.UNWRAP_MODE, bobAgree);
            simKey = (SecretKey) cipher.unwrap(wrappedKey, null, Cipher.SECRET_KEY);
            System.out.println("Bob's session key unwrapping was performed");

            /* Расшифрование бобом текста на расшифрованном симметричном ключе.
            IV передан от алисы */
            cipher = Cipher.getInstance(CipherAlgorithm, defaultProvider);
            cipher.init(Cipher.DECRYPT_MODE, simKey,
                new IvParameterSpec(iv), null);

            byte[] decryptedText = cipher
                .doFinal(encryptedText, 0, encryptedText.length);
            System.out.println("Alica's text decrypting was performed");

            // проверка результата.
            if (decryptedText.length != data.length) {
                throw new Exception("Error in crypting");
            } // if

            for (int i = 0; i < decryptedText.length; i++) {
                if (data[i] != decryptedText[i]) {
                    throw new Exception("Error in crypting");
                }
            } // for

            System.out.println("Example is passed. OK.");

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        } finally {

            if (alicaPrivate != null) {
                ((GostPrivateKey) alicaPrivate).clear();
            }

            if (bobPrivate != null) {
                ((GostPrivateKey) bobPrivate).clear();
            }

            // Удаление контейнеров.
            clear(delete);

        }
    }

    /**
     *
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        main_different_encrypt(true);
    }

    /**
     * Использование различных алгоритмов для шифрования.
     *
     * @param gost2012Enabled True, если выполнять пример с
     * применением ГОСТ 2012.
     * @throws Exception
     */
    public static void main_different_encrypt(boolean gost2012Enabled) throws Exception {

        SessionEncryptByEph sessionEncryptByEph = new SessionEncryptByEph();

        // ГОСТ Р 34.10-2001 DH EPH
        sessionEncryptByEph.execute(JCSP.PROVIDER_NAME, JCSP.PROVIDER_NAME,
                JCSP.PROVIDER_NAME, null, JCP.GOST_EL_DH_EPH_NAME, JCP.GOST_EL_DH_NAME,
                JCP.GOST_EL_SIGN_NAME, false, false);

        if (gost2012Enabled) {

            // ГОСТ Р 34.10-2012 (256) DH EPH
            sessionEncryptByEph.execute(JCSP.PROVIDER_NAME, JCSP.PROVIDER_NAME,
                    JCSP.PROVIDER_NAME, CryptParamsSpec.getInstance(CryptParamsSpec.Rosstandart_TC26_Z),
                    JCP.GOST_EPH_DH_2012_256_NAME, JCP.GOST_DH_2012_256_NAME, JCP.GOST_SIGN_2012_256_NAME,
                    false, false);

            // ГОСТ Р 34.10-2012 (512) DH EPH
            sessionEncryptByEph.execute(JCSP.PROVIDER_NAME, JCSP.PROVIDER_NAME,
                    JCSP.PROVIDER_NAME, CryptParamsSpec.getInstance(CryptParamsSpec.Rosstandart_TC26_Z),
                    JCP.GOST_EPH_DH_2012_512_NAME, JCP.GOST_DH_2012_512_NAME, JCP.GOST_SIGN_2012_512_NAME,
                    false, false);

        } // if

    }

}

