/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.EncryptDecryptData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCSP.JCSP;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import java.security.*;
import java.security.cert.X509Certificate;
import java.security.spec.AlgorithmParameterSpec;

/**
 * Класс ClassicEncryptDecryptExample реализует пример зашифрования
 * и расшифрования сообщения на ключе согласования сторон.
 *
 * 27/05/2013
 *
 */
public class ClassicEncryptDecryptExample extends EncryptDecryptData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public ClassicEncryptDecryptExample(ContainerAdapter adapter) {
        super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new ClassicEncryptDecryptThread());
    }

    /**
     * Класс CheckCertStatusThread реализует зашифрование
     * и расшифрование сообщения по классической схеме на
     * ключе согласования сторон в отдельном потоке.
     *
     */
    private class ClassicEncryptDecryptThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            // Сторона получателя.

            Logger.log("Load recipient parameters.");

            ClassicEncryptDecryptExample recipient =
                new ClassicEncryptDecryptExample(containerAdapter);

            recipient.load(true, keyStoreType, containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            PrivateKey recipientPrivateKey = recipient.getPrivateKey();
            X509Certificate recipientCert = recipient.getCertificate();
            PublicKey recipientPublicKey = recipientCert.getPublicKey();

            Logger.log(
                "Recipient certificate: " + recipientCert.getSubjectDN() +
                ", public key: " + recipientPublicKey);

            // Создание эфемерного ключа на стороне
            // отправителя.

            String ephKeyAlgorithm;
            switch (algorithmSelector.getProviderType()) {

                case pt2012Short: {
                    ephKeyAlgorithm = JCP.GOST_EPH_DH_2012_256_NAME;
                }
                break;

                case pt2012Long: {
                    ephKeyAlgorithm = JCP.GOST_EPH_DH_2012_512_NAME;
                }
                break;

                default: {
                    ephKeyAlgorithm = JCP.GOST_EL_DH_EPH_NAME;
                }

            } // switch

            KeyPairGenerator ephKpg = KeyPairGenerator.getInstance(
                ephKeyAlgorithm, JCSP.PROVIDER_NAME);

            ephKpg.initialize(getKeyParams(recipientCert));
            KeyPair ephKeyPair = ephKpg.generateKeyPair();

            Logger.log("Sender's ephemeral key was generated.");

            PrivateKey ephPrivateKey = ephKeyPair.getPrivate();
            PublicKey ephPublicKey = ephKeyPair.getPublic();

            // Генерация начальной синхропосылки для выработки
            // ключа согласования отправителем.

            Logger.log("Generate IV.");
            StringBuffer buffer = new StringBuffer();

            for (int i = 0; i < 1024; i++) {
                buffer.append(Constants.MESSAGE);
            } // for

            byte[] data = buffer.toString().getBytes();
            byte[] sv = new byte[8];
            final String MODE = "/CNT/NoPadding";

            Logger.log("Set MODE: GOST28147" + MODE);

            SecureRandom random = SecureRandom.getInstance(JCP.CP_RANDOM, JCSP.PROVIDER_NAME);
            random.nextBytes(sv);

            IvParameterSpec ivSpec = new IvParameterSpec(sv);
            Logger.log("Syncro for KeyAgreement was generated.");

            KeyAgreement keyAgree = KeyAgreement.getInstance(
                ephPrivateKey.getAlgorithm(), JCSP.PROVIDER_NAME);

            keyAgree.init(ephPrivateKey, ivSpec, null);
            keyAgree.doPhase(recipientPublicKey, true);

            SecretKey agreeKey = keyAgree.generateSecret(JCSP.GOST_CIPHER_NAME);
            Logger.log("Recipient's key agreement was performed.");

            // Зашифрование текста на ключе согласования
            // получателя.

            Cipher cipher = Cipher.getInstance(
                JCSP.GOST_CIPHER_NAME + MODE,
                JCSP.PROVIDER_NAME
            );

            cipher.init(Cipher.ENCRYPT_MODE, agreeKey,
                (AlgorithmParameterSpec) null, null);

            // Передача вектора инициализации получателю.

            byte[] iv = cipher.getIV();
            byte[] encryptedText = cipher.doFinal(data, 0, data.length);

            Logger.log("Recipient's encrypting was performed.");

            // Выработка ключа согласования получателя с тем же SV.

            keyAgree = KeyAgreement.getInstance(recipientPrivateKey
                .getAlgorithm(), JCSP.PROVIDER_NAME);

            keyAgree.init(recipientPrivateKey, ivSpec, null);
            keyAgree.doPhase(ephPublicKey, true);

            agreeKey = keyAgree.generateSecret(JCSP.GOST_CIPHER_NAME);
            Logger.log("Recipient's key agreement was performed.");

            // Расшифрование текста на ключе согласования получателя.
            // IV передан от отправителя.

            cipher = Cipher.getInstance(JCSP.GOST_CIPHER_NAME + MODE,
                JCSP.PROVIDER_NAME);

            cipher.init(Cipher.DECRYPT_MODE, agreeKey,
                new IvParameterSpec(iv), null);

            byte[] decryptedText = cipher.doFinal(encryptedText,
                0, encryptedText.length);

            Logger.log("Recipient's decrypting was performed.");

            // Проверка результата.

            if (decryptedText.length != data.length) {
                Logger.log("Error in encrypting/decrypting. Invalid length.");
                Logger.setStatusFailed();
                return;
            } // if

            for (int i = 0; i < decryptedText.length; i++) {
                if (data[i] != decryptedText[i]) {
                    Logger.log("Error in encrypting/decrypting. Invalid data.");
                    Logger.setStatusFailed();
                    return;
                } // if
            } // for

            Logger.setStatusOK();

        }

    }
}
