/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import android.content.Context;

import ru.CryptoPro.ACSPClientApp.client.example.base.GenKeyPairData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCP.KeyStore.JCPPrivateKeyEntry;
import ru.CryptoPro.JCP.params.AlgIdSpec;

import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.JCSP.Key.GostPrivateKey;
import ru.CryptoPro.JCSP.Key.GostPublicKey;
import ru.CryptoPro.JCSP.params.NameAlgIdSpec;

import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

/**
 * Класс GenKeyPairExample реализует пример генерации
 * ключевой пары и создания контейнера.
 * Ввод пин-кодов осуществляется в окно CSP, хотя можно
 * настроить пример для ввода пин-кода программно, используя
 * KeyStore.ProtectionParameters и KeyStore.setEntry().
 *
 * 27/06/2013
 *
 */
public class GenKeyPairExample extends GenKeyPairData {

    /**
     * Версия УЦ.
     */
    private CAType caType = CAType.msCA;

    /**
     * Контекст приложения.
     */
    private Context context = null;

    /**
     * Конструктор. Производит создание генератора
     * чисел.
     *
     * @param adapter Настройки примера.
     * @param caType Версия УЦ.
     * @param context Контекст приложения.
     */
    public GenKeyPairExample(ContainerAdapter adapter, CAType
        caType, Context context) {

        super(adapter);

        this.caType = caType;
        this.context = context;

    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new GenKeyPairThread());
    }

    /**
     * Класс SimpleTLSThread реализует подключение
     * apache http клиента по TLS в отдельном потоке.
     *
     */
    private class GenKeyPairThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            String keyAlgorithmName = getKeyAlgorithm(
                containerAdapter.getProviderType(),
                containerAdapter.isExchangeKey());

            Logger.log("Create key pair generator: " + keyAlgorithmName);

            KeyPairGenerator kg = KeyPairGenerator.getInstance(
                keyAlgorithmName, JCSP.PROVIDER_NAME);

            Logger.log("Store name: " + storeName +
                ".\nInitialize generator.");

            // Проинициализируем генератор именем. Если это
            // алгоритм DH, то следует указать параметры
            // дополнительно.

            AlgIdSpec params = new NameAlgIdSpec(storeName);
            kg.initialize(params);

            Logger.log("Generate key pair.");

            // Сгенерим пару ключей.

            final KeyPair keypair = kg.generateKeyPair();

            Logger.log("\tprivate key: " + keypair.getPrivate() + "\n\tparameters: " +
                ((GostPrivateKey) keypair.getPrivate()).getSpec().getParams());

            Logger.log("\n\tpublic key: " + keypair.getPublic() + "\n\tparameters: " +
                ((GostPublicKey)keypair.getPublic()).getSpec().getParams());

            // Получение самодподписанного сертификата-заглушки.

            Logger.log("Create self-signed certificate stub.");

            X509Certificate selfSignedCertStub = getSelfSignedCertStub(
                storeAlias, keypair);

            // Сохранение контейнера с сертификатом-заглушкой.

            Logger.log("Open key store and load containers.");

            KeyStore keyStore = KeyStore.getInstance(keyStoreType, JCSP.PROVIDER_NAME);
            keyStore.load(null, null);

            // В случае с УЦ 2.0 в примере далее есть авторизация по сертификату
            // и получение списка сертификатов пользователя. Для этого нужен пароль
            // к контейнеру пользователя, который никак получить нельзя, разве что
            // задав его явно, как в случае setEntry.

            if (askPinInDialog && caType != CAType.ca20) {

                // При сохранении пин-код будет запрошен в окне CSP.

                keyStore.setKeyEntry(storeAlias, keypair.getPrivate(),
                    null, new Certificate[] {selfSignedCertStub});

            } // if
            else {

                // Сохранение контейнера с паролем {@link #defaultPassword},
                // чтобы он был известен далее. Используется, в частности,
                // с примером для УЦ 2.0, так как далее есть пробная авторизация
                // по сертификату и надо знать пароль заранее, а иначе, как указав
                // его тут, его не узнать.

                Logger.log("NEW key store PASSWORD (CA20): " +
                    ( defaultPassword != null ? String.valueOf(defaultPassword) : null ));

                KeyStore.ProtectionParameter protectedParam =
                    new KeyStore.PasswordProtection(defaultPassword);

                JCPPrivateKeyEntry entry =
                    new JCPPrivateKeyEntry(keypair.getPrivate(),
                    new Certificate[] {selfSignedCertStub});

                keyStore.setEntry(storeAlias, entry, protectedParam);

            } // else

            if (caType != CAType.selfSigned) {

                // Получение из УЦ и установка сертификата в контейнер,
                // если требуется.

                final String keyStorePassword =
                    (defaultPassword != null ? String.valueOf(defaultPassword) : null);

                X509Certificate userCert = generateCertificate(caType, storeAlias,
                    keypair.getPrivate(), keypair.getPublic());

                Logger.log("Retrieved certificate:\n\tsubject: " +
                    userCert.getSubjectDN() + "\n\tserial: " +
                    userCert.getSerialNumber().toString(16));

                // Пароль не используется, т.к. он был задан в окне
                // при генерации пары. При загрузке используем провайдер
                // Java CSP.

                Logger.log("Install certificate to the store...");

                keyStore.setCertificateEntry(storeAlias, userCert);
                Logger.log("Certificate is installed.");

                // В случае УЦ 2.0 пробуем авторизоваться по сертификату
                // и выполнить простой запрос списка сертификатов.

                if (caType == CAType.ca20) {

                    Logger.log("Try to authorize by user certificate and " +
                        "load certificate list...");

                    tryToAuthorizeByCertificateAndLoadCertList(
                        keyStoreType, keyStorePassword, context);

                } // if

            } // if

            Logger.setStatusOK();

        }

    }
}
