/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.EncryptDecryptData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.params.CryptParamsSpec;
import ru.CryptoPro.JCP.spec.GostCipherSpec;

import ru.CryptoPro.JCSP.JCSP;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import java.security.*;
import java.security.cert.X509Certificate;
import java.security.spec.AlgorithmParameterSpec;

/**
 * Класс SessionEncryptDecryptExample реализует пример зашифрования
 * и расшифрования сообщения на симметричном ключе.
 *
 * 28/05/2013
 *
 */
public class SessionEncryptDecryptExample extends EncryptDecryptData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public SessionEncryptDecryptExample(ContainerAdapter adapter) {
       super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new SessionEncryptDecryptThread());
    }

    /**
     * Класс SessionEncryptDecryptThread реализует зашифрование
     * и расшифрование сообщения по на симметричном ключе в
     * отдельном потоке. Для передачи этого ключа производится
     * его зашифрование и расшифрование на ключе согласования
     * сторон.
     *
     */
    private class SessionEncryptDecryptThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            // Сторона получателя.

            Logger.log("Load recipient parameters.");

            SessionEncryptDecryptExample recipient =
                new SessionEncryptDecryptExample(containerAdapter);

            recipient.load(true, keyStoreType, containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            PrivateKey recipientPrivateKey = recipient.getPrivateKey();
            X509Certificate recipientCert = recipient.getCertificate();
            PublicKey recipientPublicKey = recipientCert.getPublicKey();

            Logger.log(
                "Recipient certificate: " + recipientCert.getSubjectDN() +
                ", public key: " + recipientPublicKey);

            // Создание эфемерного ключа на стороне
            // отправителя.

            String ephKeyAlgorithm;
            switch (algorithmSelector.getProviderType()) {

                case pt2012Short: {
                    ephKeyAlgorithm = JCP.GOST_EPH_DH_2012_256_NAME;
                }
                break;

                case pt2012Long: {
                    ephKeyAlgorithm = JCP.GOST_EPH_DH_2012_512_NAME;
                }
                break;

                default: {
                    ephKeyAlgorithm = JCP.GOST_EL_DH_EPH_NAME;
                }

            } // switch

            KeyPairGenerator ephKpg = KeyPairGenerator.getInstance(
                ephKeyAlgorithm, JCSP.PROVIDER_NAME);

            ephKpg.initialize(getKeyParams(recipientCert));
            KeyPair ephKeyPair = ephKpg.generateKeyPair();

            Logger.log("Sender's ephemeral key was generated.");

            PrivateKey ephPrivateKey = ephKeyPair.getPrivate();
            PublicKey ephPublicKey = ephKeyPair.getPublic();

            // Генерация начальной синхропосылки для выработки
            // ключа согласования отправителем.

            Logger.log("Generate IV.");
            StringBuffer buffer = new StringBuffer();

            for (int i = 0; i < 1024; i++) {
                buffer.append(Constants.MESSAGE);
            } // for

            byte[] data = buffer.toString().getBytes();
            byte[] sv = new byte[8];
            final String MODE = "/CNT/NoPadding";

            Logger.log("Set mode: GOST28147" + MODE);

            SecureRandom random = SecureRandom.getInstance(JCP.CP_RANDOM, JCSP.PROVIDER_NAME);
            random.nextBytes(sv);

            IvParameterSpec ivSpec = new IvParameterSpec(sv);
            Logger.log("Syncro for KeyAgreement was generated.");

            KeyAgreement keyAgree = KeyAgreement.getInstance(
                ephPrivateKey.getAlgorithm(), JCSP.PROVIDER_NAME);

            keyAgree.init(ephPrivateKey, ivSpec, null);
            keyAgree.doPhase(recipientPublicKey, true);

            SecretKey agreeKey = keyAgree.generateSecret(JCSP.GOST_CIPHER_NAME);
            Logger.log("Sender's key agreement was performed.");

            // Генерация симметричного ключа алисой с параметрами
            // шифрования отправителем.

            KeyGenerator keyGen = KeyGenerator.getInstance(
                JCSP.GOST_CIPHER_NAME, JCSP.PROVIDER_NAME);

            switch (algorithmSelector.getProviderType()) {

                case pt2012Short:
                case pt2012Long:
                    keyGen.init(CryptParamsSpec.getInstance(CryptParamsSpec.Rosstandart_TC26_Z));
                    break;

            } // switch

            SecretKey symKey = keyGen.generateKey();

            Logger.log("Session key was generated by sender: " +
                symKey.getAlgorithm());

            // Зашифрование текста на симметричном ключе
            // отправителя.

            Cipher cipher = Cipher.getInstance(JCSP.GOST_CIPHER_NAME + MODE, JCSP.PROVIDER_NAME);
            cipher.init(Cipher.ENCRYPT_MODE, symKey);

            // Передача вектора инициализации получателю.

            byte[] iv = cipher.getIV();
            byte[] encryptedText = cipher.doFinal(data, 0, data.length);

            Logger.log("Sender's encrypting was performed.");

            // Зашифрование симметричного ключа на ключе
            // согласования отправителя.

            cipher.init(Cipher.WRAP_MODE, agreeKey);
            byte[] wrappedKey = cipher.wrap(symKey);

            Logger.log("Sender's session key wrapping was performed.");

            // Выработка ключа согласования получателя с
            // тем же SV.

            keyAgree = KeyAgreement.getInstance(recipientPrivateKey
                .getAlgorithm(), JCSP.PROVIDER_NAME);

            keyAgree.init(recipientPrivateKey, ivSpec, null);
            keyAgree.doPhase(ephPublicKey, true);

            agreeKey = keyAgree.generateSecret(JCSP.GOST_CIPHER_NAME);
            Logger.log("Recipient's key agreement was performed.");

            // Расшифрование получателем симметричного ключа.

            cipher.init(Cipher.UNWRAP_MODE, agreeKey);
            symKey = (SecretKey) cipher.unwrap(wrappedKey, null, Cipher.SECRET_KEY);

            Logger.log("Recipient's session key unwrapping was performed: " +
                symKey.getAlgorithm());

            // Расшифрование текста на расшифрованном симметричном
            // ключе. IV передан от отправителя.

            cipher = Cipher.getInstance(JCSP.GOST_CIPHER_NAME + MODE,
                JCSP.PROVIDER_NAME);

            AlgorithmParameterSpec decryptSpec;
            switch (algorithmSelector.getProviderType()) {

                case pt2012Short:
                case pt2012Long:
                    decryptSpec = new GostCipherSpec(new IvParameterSpec(iv),
                        CryptParamsSpec.getInstance(CryptParamsSpec.Rosstandart_TC26_Z));
                    break;

                default:
                    decryptSpec = new IvParameterSpec(iv);

            } // switch

            cipher.init(Cipher.DECRYPT_MODE, symKey, decryptSpec, null);
            byte[] decryptedText = cipher.doFinal(encryptedText, 0, encryptedText.length);

            Logger.log("Recipient's decrypting was performed.");

            // Проверка результата.

            if (decryptedText.length != data.length) {
                Logger.log("Error in encrypting/decrypting. Invalid length.");
                Logger.setStatusFailed();
                return;
            } // if

            for (int i = 0; i < decryptedText.length; i++) {
                if (data[i] != decryptedText[i]) {
                    Logger.log("Error in encrypting/decrypting. Invalid data.");
                    Logger.setStatusFailed();
                    return;
                } // if
            } // for

            Logger.setStatusOK();

        }

    }

}
