/**
 * $RCSfileHttpsExample.java,v $
 * version $Revision: 36379 $
 * created 27.09.2017 13:03 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.interfaces.OnPageLoadListener;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCP.tools.Array;
import ru.CryptoPro.ssl.android.Provider;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import java.io.*;

import java.net.URL;
import java.security.KeyStore;

/**
 * Пример HttpsExample предназначен для
 * создания подключения к удаленному серверу
 * с использованием ГОСТ TLS v.1.0.
 * Клиентская аутентификация отключена.
 * Страница загружается в файл и передается
 * в главную активити для просмотра.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class HttpsExample implements Executable<Void> {

    /**
     * Адрес подключения.
     */
    public static final String URL = "https://cpca.cryptopro.ru/";

    /**
     * Страница.
     */
    public static final String PAGE = "default.htm";

    /**
     * Разделитель.
     */
    public static final String HTTP_SEPARATOR = "\r\n\r\n";

    /**
     * Слушатель событий подключения для передачи
     * в главную активити загруженной страницы.
     */
    private OnPageLoadListener onPageLoadListener;

    /**
     * Папка для сохранения загруженного файла.
     */
    private final File tempPath;

    /**
     * Хранилище доверенных сертификатов.
     */
    private final KeyStore trustStore;

    /**
     * Конструктор.
     *
     * @param trustStoreType Тип хранилища доверенных сертификатов.
     * @param tempPath Папка для сохранения загруженного файла.
     * @param trustStorePath Путь к файлу хранилища доверенных
     * сертификатов.
     * @param password Пароль хранилища доверенных сертификатов.
     * @throws Exception
     */
    public HttpsExample(String trustStoreType, File tempPath,
        String trustStorePath, char[] password) throws Exception {

        this.tempPath = tempPath;

        trustStore = KeyStore.getInstance(trustStoreType,
            BouncyCastleProvider.PROVIDER_NAME);

        trustStore.load(new FileInputStream(trustStorePath),
            password);

    }

    /**
     * Задание слушателя событий.
     *
     * @param listener слушатель.
     */
    public void setOnPageLoadListener(OnPageLoadListener listener) {
        onPageLoadListener = listener;
    }

    @Override
    public Void execute() throws Exception {

        // Защищенный контекст.

        SSLContext sslContext = createSSLContext();
        SSLSocketFactory sslFactory = sslContext.getSocketFactory();

        URL url = new URL(URL);
        String host = url.getHost();

        int port = url.getPort();
        SSLSocket sslSocket = null;

        if (port <= 0) {
            port = url.getDefaultPort();
        } // if

        try {

            // Отправка запроса и чтение ответа.

            Logger.INSTANCE.append("Creating socket...");
            sslSocket = (SSLSocket) sslFactory.createSocket(host, port);

            InputStream inputStream   = sslSocket.getInputStream();
            OutputStream outputStream = sslSocket.getOutputStream();

            String request = "GET /" + PAGE + " HTTP/1.0" + HTTP_SEPARATOR;
            Logger.INSTANCE.append("Sending request: " + request);

            outputStream.write(request.getBytes());
            outputStream.flush();

            Logger.INSTANCE.append("Decoding response...");

            String response = new String(readHeader(inputStream,
                HTTP_SEPARATOR.getBytes()));

            int fileLength = parseAnswer(response);
            byte[] bodyContent = readBody(inputStream, fileLength);

            File htmlFile = new File(tempPath, "downloaded.html");
            Array.writeFile(htmlFile, bodyContent);

            // Передача уведомления и открытие страницы.

            Logger.INSTANCE.append("Https request completed," +
                " downloaded file: " + htmlFile);

            if (onPageLoadListener != null) {
                onPageLoadListener.onPageLoad(htmlFile);
            } // if

        } finally {

            if (sslSocket != null) {

                try {
                    sslSocket.close();
                } catch (Exception e) {
                    // ignore
                }

            } // if

        }

        return null;

    }

    /**
     * Создание защищенного контекста.
     *
     * @return защищенный контекст.
     * @throws Exception
     */
    private SSLContext createSSLContext() throws Exception {

        Logger.INSTANCE.append("Create SSL context...");

        TrustManagerFactory tmf = TrustManagerFactory.getInstance(
            Provider.KEYMANGER_ALG,
            Provider.PROVIDER_NAME
        );

        tmf.init(trustStore);

        SSLContext sslContext = SSLContext.getInstance(
            Provider.ALGORITHM,
            Provider.PROVIDER_NAME
        );

        sslContext.init(null, tmf.getTrustManagers(), null);
        Logger.INSTANCE.append("SSL context created.");

        return sslContext;

    }

    /**
     * Чтение заголовков ответа сервера.
     *
     * @param in Входящий поток.
     * @param end Разделитель.
     * @return заголовки.
     * @throws IOException
     */
    private byte[] readHeader(InputStream in, byte[] end)
        throws IOException {

        Logger.INSTANCE.append("Read http headers...");

        ByteArrayOutputStream outputStream =
            new ByteArrayOutputStream();

        int conformity = 0;
        int next;

        do {

            next = in.read();
            if (next == -1) {
                throw new IOException(" Client: Error reading HTTP header");
            } // if

            outputStream.write(next);
            if (next == end[conformity]) {
                conformity++;
            } // if
            else {
                conformity = 0;
            } // else

        } while (conformity != end.length);

        return outputStream.toByteArray();

    }

    /**
     * Разбор заголовков ответа.
     *
     * @param str Заголовки ответа.
     * @return длина сообщения.
     * @throws IOException
     */
    private int parseAnswer(String str) throws IOException {

        Logger.INSTANCE.append("Parse http response...");
        final String[] split = str.split("\r\n");

        if (!split[0].equalsIgnoreCase("HTTP/1.0 200 OK") &&
            !split[0].equalsIgnoreCase("HTTP/1.1 200 OK")) {
            throw new IOException(split[0]);
        } // if

        int len = -1;

        for (int i = 1; i < split.length; i++) {
            if (split[i].startsWith("Content-Length:")) {

                final String ss = split[i].substring(
                    "Content-Length:".length()).trim();

                len = Integer.parseInt(ss);
                break;

            } // if
        } // for

        return len;

    }

    /**
     * Чтение тела ответа.
     *
     * @param in Входящий поток.
     * @param len Длина тела.
     * @return тело ответа.
     * @throws IOException
     */
    private byte[] readBody(InputStream in, int len)
        throws IOException {

        // Если есть размер сообщения (прочитан из хидеров),
        // то используем его.

        Logger.INSTANCE.append("Read http body...");
        if (len > 0) {

            final byte[] buf = new byte[len];
            int next;
            int pos = 0;

            while (pos != len) {
                next = in.read();

                if (next == -1) {
                    throw new IOException(" Error reading HTTP body");
                } // if

                buf[pos++] = (byte) next;
            } // while

            return buf;
        } // if
        // Если размера нет (такое бывает), то читаем,
        // пока не получим конец файла.
        else {

            ByteArrayOutputStream buf = new ByteArrayOutputStream();
            while (true) {

                int next = in.read();
                if (next == -1) {

                    // Ничего не прочитали, сразу конец файла.
                    if (buf.size() == 0) {
                        throw new IOException(" Error reading HTTP body");
                    } // if

                    break;
                } // if

                buf.write((byte) next);
            } // while

            return buf.toByteArray();

        } // else

    }

}
