package cmsutil.asn1;

import cmsutil.tools.ProviderUtil;
import ru.CryptoPro.Crypto.CryptoProvider;
import cmsutil.asn1.base.ASNCommon;
import cmsutil.asn1.base.ASNInteger;
import cmsutil.asn1.base.ASNSequence;
import cmsutil.asn1.base.ASNSet;
import cmsutil.asn1.encrypted.CipherProcessor;
import cmsutil.asn1.encrypted.EncryptedContentInfo;
import cmsutil.asn1.kari.RecipientCertInfo;
import cmsutil.asn1.kari.RecipientInfos;
import cmsutil.tools.OID;

import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;

/**
 * Класс, осуществляющий кодирование структуры EnvelopedData.
 */
public class EnvelopedData extends ASNSequence {
    /** Версия структуры */
    private ASNInteger version;
    /** Структура RecipientInfos */
    private RecipientInfos recipientInfos;
    /** Структура EncryptedContentInfo*/
    private EncryptedContentInfo encryptedContentInfo;

    private final static OID cipherParamOID = OID.OID_Crypt_VerbaO;

    /**
     * Конструктор, используемый при создании сообщения CMS Enveloped.
     * @param textLength Длина открытого текста
     * @param recipientCertificate Сертификат открытого ключа получателя.
     * @throws CMSCryptographyException Ошибка криптографической операции.
     * @throws ASNDecodeException Ошибка декодирования.
     */
    public EnvelopedData(long textLength, X509Certificate recipientCertificate, String provider)
        throws CMSCryptographyException, ASNDecodeException
    {
        version = new ASNInteger(new BigInteger("2"));
        SecretKey cek;
        try {
            String enc_provider = ProviderUtil.findEncryptionProvider(provider);
            KeyGenerator keyGenerator = KeyGenerator.getInstance(CryptoProvider.GOST_CIPHER_NAME, enc_provider);
            cek = keyGenerator.generateKey();
        }
        catch (NoSuchAlgorithmException e) {
            throw new CMSCryptographyException(e.getMessage());
        }
        catch (NoSuchProviderException e) {
            throw new CMSCryptographyException(e.getMessage());
        }
        recipientInfos = new RecipientInfos(cek, recipientCertificate, provider);
        encryptedContentInfo = new EncryptedContentInfo(textLength, cek, cipherParamOID, provider);
        subs = new ArrayList<ASNCommon>(3);
        subs.add(version);
        subs.add(recipientInfos);
        subs.add(encryptedContentInfo);
        encode();
    }

    /**
     * Конструктор, используемый при разборе сообщения CMS Enveloped.
     * @param structure ASN1-структура, соответствующая классу.
     * @param recipientPrivateKey Секретный ключ получателя ГОСТ Р 34.10-2001.
     * @throws ASNDecodeException Ошибка декодирования.
     * @throws CMSCryptographyException Ошибка криптографической операции.
     */
    public EnvelopedData(ASNSequence structure, PrivateKey recipientPrivateKey, String provider)
            throws ASNDecodeException, CMSCryptographyException
    {
        this.subs = structure.getSubStructures();
        this.encodedValue = structure.getEncoded();
        this.realInternalLength = structure.getRealInternalLength();
        this.realEncodedLength = structure.getRealEncodedLength();
        this.virtualEncodedLength = structure.getVirtualEncodedLength();
        this.virtualInternalLength = structure.getVirtualInternalLength();
        if (!checkConsist())
            throw new ASNDecodeException("Structure EnvelopedData is corrupted!");
        this.version = (ASNInteger)subs.get(0);
        if (!this.version.getValue().equals(BigInteger.valueOf(2)))
            throw new ASNDecodeException("Structure EnvelopedData is corrupted!");
        this.recipientInfos = new RecipientInfos((ASNSet)subs.get(1), recipientPrivateKey, provider);
        this.encryptedContentInfo = new EncryptedContentInfo((ASNSequence)subs.get(2), this.recipientInfos.getCEK(), provider);
    }

    /**
     * Метод, проверяющий корректность структуры.
     * @return true, тогда и только тогда, когда количество подструктур и их содержимое корректно.
     */
    @Override
    protected boolean checkConsist() {
        return (subs != null) && (subs.size() == 3) && (subs.get(0) instanceof ASNInteger) &&
                (subs.get(1) instanceof ASNSet) && (subs.get(2) instanceof ASNSequence);
    }

    /**
     * Метод, получающий высокоуровневый объект шифратора.
     * @return Высокоуровневый объект шифратора.
     */
    public CipherProcessor getCipherProcessor() {
        return encryptedContentInfo.getCipherProcessor();
    }

    /**
     * Метод, получающий информацию о сертификате получателя.
     * @return Информацию о сертификате получателя.
     */
    public RecipientCertInfo getRecipientCertInfo() {
        return recipientInfos.getRecipientCertInfo();
    }
}
