/**
 * $RCSfileSecureConnectionManager.java,v $
 * version $Revision: 36379 $
 * created 16.08.2016 10:43 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package tls_proxy;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.KeyStore.StoreInputStream;

import ru.CryptoPro.ssl.Provider;
import javax.net.ssl.*;

import java.io.FileInputStream;
import java.security.KeyStore;

/**
 * Класс SecureConnectionManager предназначен
 * для создания защищенного сокета по адресу
 * и другим параметрами.
 *
 * @author Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class SecureConnectionManager {

    /**
     * Адрес подключения.
     */
    private final Address address;

    /**
     * Фабрика защищенных сокетов.
     */
    private final SSLSocketFactory sslSocketFactory;

    /**
     * Конструктор.
     * Выполняет создание защищенного контекста
     * и инициализацию фабрики сокетов.
     *
     * @param address Адрес подключения.
     */
    public SecureConnectionManager(Address address) {

        if (address == null) {
            throw new IllegalArgumentException("Address is null.");
        } // if

        this.address = address;

        try {
            this.sslSocketFactory = createSSLContext().getSocketFactory();
        } catch (Exception e) {
            throw new IllegalArgumentException(e);
        }

    }

    /**
     * Создание защищенного контекста.
     *
     * @return защищенный контекст.
     * @throws Exception
     */
    private SSLContext createSSLContext() throws Exception {

        KeyManagerFactory kmf = null;

        String certStoreProvider = ConfigReader.INSTANCE.getCertStoreProvider();
        String certStoreType = ConfigReader.INSTANCE.getCertStoreType();

        String certStorePath = ConfigReader.INSTANCE.getCertStorePath();
        String certStorePassword = ConfigReader.INSTANCE.getCertStorePassword();

        String provider = ConfigReader.INSTANCE.getProvider();
        String protocol = ConfigReader.INSTANCE.getProtocol();

        if (certStoreType == null || certStoreType.isEmpty()) {
            certStoreType = JCP.CERT_STORE_NAME;
        } // if

        MainLogger.info("Trust store parameters:"
            + "\n\tprovider: " + certStoreProvider
            + "\n\ttype: " + certStoreType
            + "\n\tpath: " + certStorePath
            + "\n\tclient auth: " + address.isClientAuthEnabled()
            + "\n\t...");

        KeyStore trustStore;

        if (certStoreProvider != null && !certStoreProvider.isEmpty()) {
            trustStore = KeyStore.getInstance(
                certStoreType, certStoreProvider);
        } // if
        else {
            trustStore = KeyStore.getInstance(certStoreType);
        } // else

        trustStore.load(new FileInputStream(certStorePath),
            certStorePassword.toCharArray());

        if (address.isClientAuthEnabled()) {

            String keyType = address.getKeyType();
            String keyAlias = address.getKeyAlias();
            String keyPassword = address.getKeyPassword();

            MainLogger.info("Key store parameters:"
                + "\n\tprovider: " + provider
                + "\n\ttype: " + keyType
                + "\n\talias: " + keyAlias
                + "\n\t...");

            KeyStore keyStore;

            if (provider != null && !provider.isEmpty()) {
                keyStore = KeyStore.getInstance(keyType, provider);
            } // if
            else {
                keyStore = KeyStore.getInstance(keyType);
            } // else

            if (keyAlias != null && !keyAlias.isEmpty()) {
                keyStore.load(new StoreInputStream(keyAlias), null);
            } // if
            else {
                keyStore.load(null, null);
            } // else

            kmf = KeyManagerFactory.getInstance(Provider.KEYMANGER_ALG);
            kmf.init(keyStore, keyPassword.toCharArray());

        } // if

        TrustManagerFactory tmf = TrustManagerFactory.getInstance(Provider.TRUSTMANGER_ALG);
        tmf.init(trustStore);

        MainLogger.info("Creating secure context (protocol: "
            + protocol + ")...");

        SSLContext sslCtx = SSLContext.getInstance(protocol);
        sslCtx.init(kmf != null ? kmf.getKeyManagers() : null, tmf.getTrustManagers(), null);

        MainLogger.info("Secure context completed.");
        return sslCtx;

    }

    /**
     * Получение фабрики сокетов.
     *
     * @return фабрика сокетов.
     */
    public SSLSocket getSSLSocket() throws Exception {
        return (SSLSocket) sslSocketFactory.createSocket(address.getHost(), address.getPort());
    }

}
